@@ -323,7 +323,7 @@ gedit_document_init (GeditDocument *document)
 				gedit_encoding_get_charset (enc));
 
 	gedit_document_set_max_undo_levels (document,
-					    gedit_prefs_manager_get_undo_actions_limit ());    
+					    gedit_prefs_manager_get_undo_actions_limit ());
 
 	gtk_source_buffer_set_check_brackets (GTK_SOURCE_BUFFER (document), FALSE);
 }
@@ -639,7 +639,7 @@ gedit_document_get_uri (const GeditDocument* doc)
 	{
 		gchar *res;
 
-		res = eel_format_uri_for_display (doc->priv->uri);
+		res = gnome_vfs_format_uri_for_display (doc->priv->uri);
 		g_return_val_if_fail (res != NULL, g_strdup (_("Invalid file name")));
 		
 		return res;
@@ -1366,6 +1366,30 @@ gedit_document_is_untouched (const GeditDocument *doc)
 		(!gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (doc)));
 }
 
+/* FIXME: Remove this function when gnome-vfs will add an equivalent public
+   function - Paolo (Mar 05, 2004) */
+static gchar *
+get_slow_mime_type (const char *text_uri)
+{
+	GnomeVFSFileInfo *info;
+	char *mime_type;
+	GnomeVFSResult result;
+
+	info = gnome_vfs_file_info_new ();
+	result = gnome_vfs_get_file_info (text_uri, info,
+					  GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
+					  GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE |
+					  GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+	if (info->mime_type == NULL || result != GNOME_VFS_OK) {
+		mime_type = NULL;
+	} else {
+		mime_type = g_strdup (info->mime_type);
+	}
+	gnome_vfs_file_info_unref (info);
+
+	return mime_type;
+}
+
 static void
 gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 {
@@ -1382,8 +1406,6 @@ gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 
 	if (doc->priv->uri != NULL)
 		g_free (doc->priv->uri);
-		
-	g_return_if_fail (uri != NULL);
 
 	doc->priv->uri = g_strdup (uri);
 
@@ -1414,7 +1436,7 @@ gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 	{
 		gchar *mime_type;
 
-		mime_type = gnome_vfs_get_mime_type (uri);
+		mime_type = get_slow_mime_type (uri);
 	
 		if (mime_type != NULL)
 		{
@@ -2388,7 +2410,12 @@ gedit_document_find (GeditDocument* doc, const gchar* str, gint flags)
 					gtk_text_iter_ends_word (&match_end);
 
 				if (!found) 
-					iter = match_end;
+				{
+					if (GEDIT_SEARCH_IS_BACKWARDS (flags))
+						iter = match_start;
+					else
+						iter = match_end;
+				}
 			}
 			else
 				break;
@@ -2522,8 +2549,6 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (replace != NULL);	
 
-	converted_replace = gedit_utils_convert_search_text (replace);
-
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
                                     &iter,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
@@ -2543,6 +2568,8 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 	
 	gtk_text_iter_order (&sel_bound, &iter);		
 
+	converted_replace = gedit_utils_convert_search_text (replace);
+
 	gedit_document_begin_user_action (doc);
 
 	gtk_text_buffer_delete (GTK_TEXT_BUFFER (doc),
@@ -2564,6 +2591,7 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 	doc->priv->last_replace_text = g_strdup (replace);
 
 	gedit_document_end_user_action (doc);
+
 	g_free (converted_replace);
 }
 
@@ -2675,7 +2703,8 @@ gedit_document_set_language (GeditDocument *doc, GtkSourceLanguage *lang)
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
 	if (lang != NULL)
-		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc), TRUE);
+		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc),
+						 gedit_prefs_manager_get_enable_syntax_highlighting ());
 	else
 		gtk_source_buffer_set_highlight (GTK_SOURCE_BUFFER (doc), FALSE);
 
